#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "cnt-flow.h"


void FlowCounterTimer::expire(Event*)
{
      t_->RemoveOld();
}



FlowCounter::FlowCounter() : del_timer(this)
{
      pNodeHead = NULL;
      count = 0;

      del_timer.resched(RM_INTERVAL);
}

void FlowCounter::Reset()
{
      NodeType *pNode, *pNodeDel;

      pNode = pNodeHead;
      while(pNode != NULL) {
          pNodeDel = pNode;
	  pNode = pNode->pNode;
	  free(pNodeDel);
      }

      pNodeHead = NULL;
      count = 0;
      
      del_timer.resched(RM_INTERVAL);
}


void FlowCounter::UpdateCount(ItemType item)
{
      NodeType *pNodeCurrent;

      if(SearchItem(item, &pNodeCurrent)) {
	    pNodeCurrent->item.time = item.time;
      }
      else {
            InsertItem(item, pNodeCurrent);
            count++;
      }
}


int FlowCounter::SearchItem(ItemType item, NodeType **pNodeCurrent)
{
    NodeType *pNode, *pNodeSave;

    pNodeSave = NULL;
    for (pNode = pNodeHead; pNode != NULL; pNode = pNode->pNode) {     
	  if(pNode->item.des > item.des) {
	        *pNodeCurrent = pNodeSave;
		return(0);
	  }
	  else if(pNode->item.des == item.des) { 
	        if(pNode->item.fid > item.fid) {
		      *pNodeCurrent = pNodeSave;
		      return(0);
		}
		else if(pNode->item.fid == item.fid) {
		      *pNodeCurrent = pNode;
		      return(1);
		}
	  }

	  pNodeSave = pNode;
    }

    *pNodeCurrent = pNodeSave;
    return(0);
}


void FlowCounter::InsertItem(ItemType item, NodeType *pNodeInsert)
{
    NodeType *pNode, *pNodeTmp;

    if ((pNode = MakeNode(&item)) == NULL) {
        printf("Critical Error: out of memory space!!!\n");
        exit(1);
    }

    // insert at the front of LL
    if (pNodeInsert == NULL) {
	pNodeTmp = pNodeHead;
	pNodeHead = pNode;
	pNode->pNode = pNodeTmp;
    }
    // insert at middle or end of LL
    else {
        pNode->pNode = pNodeInsert->pNode;
	pNodeInsert->pNode = pNode;
    }
}

NodeType* FlowCounter::MakeNode(ItemType *item)
{
    NodeType *pNode;

    if ((pNode = (NodeType *)malloc(sizeof(NodeType))) != NULL) {
        memcpy(&(pNode->item), item, sizeof(ItemType));
        pNode->pNode = NULL;
    }
    return(pNode);
}

void FlowCounter::RemoveOld()
{
    NodeType *pNode, *pNodeSave, *pNodeDel;

    double local_time = Scheduler::instance().clock();

    pNodeSave = pNodeHead;
    pNode = pNodeHead;
 
    while(pNode != NULL) {
        if((local_time - pNode->item.time) > RM_INTERVAL) {

	    pNodeDel  = pNode;

	    if(pNode == pNodeHead) {
	        pNodeHead = pNode->pNode;
		pNodeSave = pNode->pNode;
		pNode     = pNode->pNode;
	    }
	    else {
	        pNodeSave->pNode = pNode->pNode;
		pNode = pNode->pNode;
	    }

	    free(pNodeDel);
	    count--;
	}
	else {
	    pNodeSave = pNode;
	    pNode = pNode->pNode;
	}
    }

    //printf("Garbage Collection: %f  (%d)\n", local_time, count);

    del_timer.resched(RM_INTERVAL);
}




void FlowCounter::PrintLL()
{
    NodeType *pNode;

    for (pNode = pNodeHead; pNode != NULL; pNode = pNode->pNode)     
          printf("%d.%d(%.2f) -> ", pNode->item.des, pNode->item.fid, pNode->item.time);
    printf("\n");
}


/*
main()
{
  FlowCounter counter;
  ItemType item;

  counter.PrintLL();
  printf("Count = %d\n\n", counter.Count());

  while(1) {
    printf("des & fid = ");
    scanf("%d %d %lf", &item.des, &item.fid, &item.time);
    printf("Input = %d %d %.2f\n\n", item.des, item.fid, item.time);

    if(item.des == -1) {
      printf("Reset!\n");
      counter.Reset();
    }
    else if(item.des == -2) {
      counter.RemoveOld();
    }
    else {
      counter.UpdateCount(item);
    }

    counter.PrintLL();
    printf("Count = %d\n\n", counter.Count());
  }
}

*/



