#include "config.h"
#include "timer-handler.h"

#define RM_INTERVAL 0.3   // Old item removal Interval (Sec)


class FlowCounter;


// FlowCounterTimer is used to delete old flows at
// a predefined time interval 
class FlowCounterTimer : public TimerHandler {
 public:
        FlowCounterTimer(FlowCounter* t) : TimerHandler(), t_(t) {}
        inline virtual void expire(Event*);
 protected:
        FlowCounter* t_;
};



// item definition
typedef struct itemtype {
      nsaddr_t des;
      int      fid;
      double   time;
} ItemType;

// linked list node definition
typedef struct nodetag {
      ItemType item;
      struct nodetag *pNode;
} NodeType;


// Flow Counter
class FlowCounter {
 public:
      FlowCounter();
      void Reset();
      inline int Count() const { return (count); }
      void UpdateCount(ItemType item);
      void RemoveOld();
      void PrintLL();

 protected:
      int SearchItem(ItemType item, NodeType **pNodeCurrent);
      void InsertItem(ItemType item, NodeType *pNodeInsert);
      NodeType* MakeNode(ItemType *item);

      int count;            // count of flows
      NodeType* pNodeHead;  // head of linked list

 private:
      FlowCounterTimer del_timer;  // Timer to delete old flows

};

